/*
    Title:      Working directory manipulation

	Copyright (c) 2000
		Cambridge University Technical Services Limited

	Copying and use of this software is subject to the terms of the
	Poly/ML licence.
*/

#if defined(WINDOWS_PC)
/* PC version implemented OK - 09/02/96 */

#include <windows.h>
#include <errno.h>
#include "diagnostics.h" /* for Crash */

void get_working_directory(char *buffer)
{
   if (GetCurrentDirectory(MAX_PATH+1,buffer) == 0)
  {
     Crash("get_working_directory: GetCurrentDirectory failed, errno = %i",
     GetLastError());
  }
}

void set_working_directory(const char *buffer)
{
  if (SetCurrentDirectory(buffer) == FALSE)
  {
     Crash("set_working_directory: SetCurrentDirectory failed for directory %s, errno = %i,", 
           buffer, GetLastError());
  }
}

static char home_directory[MAX_PATH+1];

void init_working_directory(void)
{
  get_working_directory(home_directory);
}

void reset_working_directory(void)
{
  set_working_directory(home_directory);
}


#else /* UNIX */

#include <sys/param.h>
#include <unistd.h>
#include <errno.h>
#include "diagnostics.h" /* for Crash */

void get_working_directory(char *buffer)
{
  if (getcwd(buffer,MAXPATHLEN+1) == (char *)NULL)
  {
     Crash("get_working_directory: getcwd failed, errno = %i,", errno);
  }
}

void set_working_directory(const char *buffer)
{
  if (chdir(buffer) != 0)
  {
     Crash("set_working_directory: chdir failed for directory %s, errno = %i,", 
           buffer, errno);
  }
}


static char home_directory[MAXPATHLEN+1];

void init_working_directory(void)
{
  get_working_directory(home_directory);
}

void reset_working_directory(void)
{
  set_working_directory(home_directory);
}

#endif /* not PC */





