/*
    Title:      Data structures shared between basioio.c and network.c.

	Copyright (c) 2000 David C. J. Matthews
	Portions of this code are derived from the original stream io
	package copyright CUTS 1983-2000.

	Copying and use of this software is subject to the terms of the
	Poly/ML licence.
*/

#ifndef IO_INTERNAL_H
#define IO_INTERNAL_H


#define IO_BIT_OPEN			1
#define IO_BIT_READ			2
#define IO_BIT_WRITE		4
#define IO_BIT_DIR			8 /* Is it a directory entry? */
#define	IO_BIT_SOCKET		16 /* Is it a socket? */
/* IO_BIT_NONBLOCKING is set on a socket if the user requested
   non-blocking mode.  All sockets are set to non-blocking mode
   so this indicates whether EWOULDBLOCK is passed back to the caller. */
#define IO_BIT_NONBLOCKING	32
#define IO_BIT_INPROGRESS	64 /* "connect" in progress on socket. */

#ifdef WINDOWS_PC

#define IO_BIT_PIPE			128
#define IO_BIT_DEV			256
#define IO_BIT_CONSOLE		512

#ifdef USEWINSOCK2
#include <winsock2.h>
#else
#include <winsock.h>
#endif
#else
typedef int SOCKET;
#include <dirent.h>
#endif

typedef struct basic_io_struct
{
	word *token; /* pointer into ML heap */
	int ioBits; /* Flag bits */
	union {
		int	ioDesc; /* File descriptor. */
#ifdef WINDOWS_PC
		struct {
			HANDLE	hFind; /* FindFirstFile handle */
			WIN32_FIND_DATA lastFind;
			int	fFindSucceeded;
		} directory;
		SOCKET sock;
#else
#define	sock ioDesc
		DIR	*ioDir; /* Directory entry. */
#endif
	} device;
	char	lookAheadChar; /* Temporarily.  This is needed for
							  the old IO functions. */
} IOSTRUCT, *PIOSTRUCT;


#define isOpen(s)	((s)->ioBits & IO_BIT_OPEN)
#define isRead(s)	((s)->ioBits & IO_BIT_READ)
#define isWrite(s)	((s)->ioBits & IO_BIT_WRITE)
#define isDirectory(s)	((s)->ioBits & IO_BIT_DIR)
#define isSocket(s)	((s)->ioBits & IO_BIT_SOCKET)
#define isNonBlocking(s)	((s)->ioBits & IO_BIT_NONBLOCKING)

#ifdef WINDOWS_PC
#define isPipe(s)	((s)->ioBits & IO_BIT_PIPE)
#define isDevice(s)	((s)->ioBits & IO_BIT_DEV)
#define isConsole(s)	((s)->ioBits & IO_BIT_CONSOLE)
#endif


extern PIOSTRUCT get_stream(Handle stream);
extern Handle make_stream_entry(void);
extern void free_stream_entry(int stream_no);
extern void close_stream(PIOSTRUCT str);

extern PIOSTRUCT basic_io_vector;

extern int emfileFlag;
#endif
