/*
    Title: 	proper_io.h

	Copyright (c) 2000
		Cambridge University Technical Services Limited

	Copying and use of this software is subject to the terms of the
	Poly/ML licence.
*/

#ifndef _PROPER_IO_H_DEFINED
#define _PROPER_IO_H_DEFINED 1

#include <sys/types.h>
#include <stdio.h>

typedef struct stat statstruct;
int proper_stat(char *, statstruct *);
int proper_lstat(char *, statstruct *);

#if defined(HPUX) && !defined(HPUX64)
/* The HPUX 9 kernel allows I/O calls to be interrupted, 
   but not restarted. We certainly don't want that, so we'll
   have to replace them all here.
*/

/* Level 2 (file descriptor) functions */
int proper_open(const char *, int, mode_t);
int proper_close(int);
int proper_dup2(int, int);
int proper_ioctl(int, int, long *);
int proper_fcntl(int, int, int);
int proper_read(int, void *, size_t);
int proper_write(int, const void *, size_t);

/* Level 3 (stream) functions */
FILE *proper_fopen(const char *, const char*);
int proper_fclose(FILE *);
int proper_fflush(FILE *);
int proper_fprintf(FILE *, const char *, ...);
int proper_printf(const char *, ...);

int proper_getc(FILE *);
#define proper_ungetc(c, stream) ungetc(c, stream)
int proper_putc(int, FILE *);
int proper_fputs(char *, FILE *);

/* A useful function! */
int proper_fwrite_chars(const void *, size_t, FILE *);

/* We don't have "proper" versions of fprintf etc. yet */

#else /* not HPUX 9 */

/* Level 2 (file descriptor) functions */
#define proper_open(path,flag,mode) open(path,flag,mode)
#define proper_close(fildes) close(fildes)
#define proper_dup2(fildes,fildes2) dup2(fildes,fildes2)
#define proper_ioctl(fildes,request,arg) ioctl(fildes,request,arg)
#define proper_fcntl(fildes,request,arg) fcntl(fildes,request,arg)
#define proper_read(fildes,buf,nbyte) read(fildes,buf,nbyte)
#define proper_write(fildes,buf,nbyte) write(fildes,buf,nbyte)

/* Level 3 (stream) functions */
#define proper_fopen(pathname,type) fopen(pathname,type)
#define proper_fclose(stream) fclose(stream)
#define proper_fflush(stream) fflush(stream)
#define proper_fprintf fprintf
#define proper_printf printf

#define proper_getc(stream) getc(stream)
#define proper_ungetc(c, stream) ungetc(c, stream)
#define proper_putc(c,stream) putc(c,stream)
#define proper_fputs(s,stream) fputs(s,stream)

#define proper_fwrite_chars(ptr,nchars,stream) (int)fwrite(ptr,1,nchars,stream)


#endif /* not HPUX */

#endif /* _PROPER_IO_H_DEFINED */
